<?php
/**
 *
 * @copyright &copy; 2010 The Open University
 * @author a.j.forth@open.ac.uk
 * @license http://www.gnu.org/copyleft/gpl.html GNU Public License
 * @package dataplus
 */

class dataplus_file_helper {
    private $path;
    private $image_path;
    private $file_path;
    private $temp_path;
		
    public function __construct($course_id, $mod_inst_id) {
        // the course directory exists, create if not
        $path = make_mod_upload_directory($course_id);

        //check the dataplus subdir exists, create if not
        $path .= '/dataplus';

        $this->create_dir($path);

        //check the module instance subdirectory exists, create if not
        $path .= '/' . $mod_inst_id;

        $this->create_dir($path);

        //Set the the path variable to the module subdir
        $this->path = $path;

        //check the subdirs for images and files exist, create if not
        $this->image_path = $path . '/images';

        $this->create_dir($this->image_path);

        $this->file_path = $path . '/files';
			
        $this->create_dir($this->file_path);
        
        $this->temp_path = $path . '/temp';
            
        $this->create_dir($this->temp_path);
        
        $this->clear_temp();
    }

		
    //return the module instance dir path
    public function get_path(){
			
        return $this->path;
			
    }
		
		
    //return the module instance dir path relative to the dataroot
    public function get_path_relative(){		
        global $CFG;

        return str_replace($CFG->dataroot,'',$this->path);
			
    }
		

    //return the path for storing images for this module instance
    public function get_image_path(){
			
        return $this->image_path;
			
    }

		
    //return the relative image path
    public function get_image_path_relative(){	
        global $CFG;

        return str_replace($CFG->dataroot,'',$this->image_path);
			
    }		
		

    //return the path for storing files for this module instance
    public function get_file_path(){
			
        return $this->file_path;
			
    }
		

    //return the relative file path
    public function get_file_path_relative(){
        global $CFG;

        return str_replace($CFG->dataroot,'',$this->file_path);

    }			


    //check the temp directory for this instance exists, if not create it and return the path
    public function get_temp_path(){
		$temp_path = $this->path . '/temp';

        $this->create_dir($temp_path);

        return $temp_path;
			
    }


    //return the relative temp path
    public function get_temp_path_relative(){
        global $CFG;

        return str_replace($CFG->dataroot,'',$this->get_temp_path());
			
    }		


    //clear the temp directory
    public function clear_temp(){
        global $CFG;
            
        require_once($CFG->libdir.'/filelib.php');
            
        $temp_path = $this->get_temp_path();
        
        $d = dir($temp_path);
        
        while (false !== ($entry = $d->read())) {
            if($entry == '.' || $entry == '..'){
                continue;
            }
            
            $path = $temp_path . '/' . $entry;
            
            if(filectime($path) < (time() - 1200)){
                $result = fulldelete($path);
                
                if(!$result){
                    return false;
                }
            }
        }
        
        return true;
    }
    
    
    //check the tozip directory for this instance exists, if not create it and return the path
    public function get_tozip_path(){
        global $USER;
        
        $path = $this->path . '/tozip' . $USER->id;

        $this->create_dir($path);

        return $path;
            
    }
    
    
    //check the tozip image directory for this instance exists, if not create it and return the path
    public function get_tozip_images_path(){
        $path = $this->get_tozip_path() . '/images';

        $this->create_dir($path);

        return $path;
            
    }   

    
    //check the tozip image directory for this instance exists, if not create it and return the path
    public function get_tozip_files_path(){
        $path = $this->get_tozip_path() . '/files';

        $this->create_dir($path);

        return $path;
            
    } 


    /**
     * resolves and returns the path for copying to.  
     *
     * @param param string $type = 'image' or 'file'.  If null, returns the module instance dir
     */ 
    public function get_copy_path($type = null){

        if($type == 'image'){
            return $this->get_image_path();
        }
        else if ($type == 'file'){
            return $this->get_file_path();
        }
        else if ($type == 'temp'){
            return $this->get_temp_path();
        }
        else if ($type == 'tozip'){
            return $this->get_tozip_path();
        }
        else if ($type == 'tozip-image'){
            return $this->get_tozip_images_path();
        }    
        else if ($type == 'tozip-file'){
            return $this->get_tozip_files_path();
        }               

        return $this->get_path();

    }
		
		
    /**
     * Copies the temp dir to the module instance dir, or subdirs for files or images
     *
     * @param string $destination, if 'image' or 'file', only copy the image or file subdirs
     * @param array $exclude = array of filenames to exclude from a copy
     */	
    public function copy_temp($destination = null, $exclude = null){
			
        $path = $this->get_copy_path($destination);
			
        $temp_path = $this->get_temp_path();

        $this->copy_dir($temp_path,$path,$exclude);
			
    }
		
		
    /**
     * Copy a directory including any subdirs
     *
     * @param string $from = the source directory
     * @param string $to = the destination directory
     * @param array $exclude = array of filenames to exclude from the copy
     * @return unknown
     */
    public function copy_dir($from,$to,$exclude = array()){
			
        $hande = opendir($from);

        while(false !== ($file = readdir($hande))) {
            if($file == "." || $file == ".." || in_array($file,$exclude)){
                continue;
            }
            else if(is_dir($from . '/' . $file)){
                $dir = $to . '/' . $file;
					
                if(file_exists($dir)){
                    fulldelete($dir);
                }
					
                mkdir($dir);
                $this->copy_dir($from . '/' . $file, $dir);
            }
            else {
                if(!copy($from . '/' . $file, $to . '/' . $file)){
                    return false;
                }
            }
        }
			
        return true;
    }
		
		
    /**
     * copy a file to the temp directory
     *
     * @param string $filename = name of the file to 
     * @param string $source = image or file
     */
    public function copy_to_temp($filename,$source=null){

    	return $this->copy($filename,$source,'temp');

    }
    
    
    /**
     * copy a file to the root dataplus directory
     *
     * @param string $filename = name of the file to 
     * @param string $source = image, file or temp
     * @param $newfilename = name of copy
     * @param string $destination = image, file or temp
     */
    public function copy($filename=null,$source=null,$newfilename = null,$destination=null){
            
        $path = $this->get_copy_path($source);
        
        if(!empty($filename)){
            $path = $path . '/' . $filename;
        }
            
        $dest = $this->get_copy_path($destination);

        if(!empty($filename)){
            if(!empty($newfilename)){
                $dest = $dest . '/' . $newfilename;
            }
            else {
                $dest = $dest . '/' . $filename;
            }
        }
        
        
        if(empty($newfilename)){
            $newfilename = $filename;
        }

        if(is_dir($path)){
            return $this->copy_dir($path,$dest);
        }
        else {        
            return copy($path, $dest);
        }
    }
		

    /**
     * create a directory if it doesn't exist.
     *
     * @param string $path, path for the directory
     * @return unknown
     */
    public function create_dir($path){
        if(!file_exists($path)){
            return mkdir($path);
        }

        return false;
    }


    /**
     * delete all the files referenced in a database column for files or images (used when such a column is deleted or the type is changed)
     *
     * @param string $col_name, name of the column containing the file references
     * @param string $type, image or file
     */
    public function delete_column_files($col_name,$type){
        global $dataplus_db;  			

        $columns = array($col_name);
	    			
        $results = $dataplus_db->query_dataplus_database($columns); 			

        foreach($results as $result){
            if(!empty($result->$col_name)){
                $this->delete_file($result->$col_name,$type);
            }
        }
    }
		
		
    /**
     * delete a supporting file or image
     *
     * @param string $name, the filename
     * @param string $type, image or file
     * @return unknown
     */
    public function delete_file($name,$type){

        if(empty($name)){
            return;
        }

        if($type == 'image'){
            $path = $this->get_image_path();
        }
        else {
            $path = $this->get_file_path();
        }

        return fulldelete($path . '/' . $name);
    }
}
?>